@@ -1,4 +1,3 @@
-import argparse
 import dataclasses
 import re
 import typing
@@ -18,7 +17,6 @@ class Status:
     SENSITIVE_INPUT = 11
 
     SUCCESS = 20
-    SUCCESS_END_OF_SESSION = 21
 
     REDIRECT_TEMPORARY = 30
     REDIRECT_PERMANENT = 31
@@ -36,11 +34,8 @@ class Status:
     BAD_REQUEST = 59
 
     CLIENT_CERTIFICATE_REQUIRED = 60
-    TRANSIENT_CERTIFICATE_REQUESTED = 61
-    AUTHORISED_CERTIFICATE_REQUIRED = 62
-    CERTIFICATE_NOT_ACCEPTED = 63
-    FUTURE_CERTIFICATE_REJECTED = 64
-    EXPIRED_CERTIFICATE_REJECTED = 65
+    CERTIFICATE_NOT_AUTHORISED = 61
+    CERTIFICATE_NOT_VALID = 62
 
 
 class Request:
@@ -54,7 +49,7 @@ class Request:
 
         url_parts = urlparse(self.url)
         if not url_parts.hostname:
-            raise ValueError("URL must contain a `hostname` part")
+            raise ValueError("Missing hostname component")
 
         if not url_parts.scheme:
             # If scheme is missing, infer it to be gemini://
@@ -62,6 +57,10 @@ class Request:
         else:
             self.scheme = url_parts.scheme
 
+        # gemini://username@host/... is forbidden by the specification
+        if self.scheme == "gemini" and url_parts.username:
+            raise ValueError("Invalid userinfo component")
+
         self.hostname = url_parts.hostname
         self.port = url_parts.port
 
@@ -126,7 +125,7 @@ class JetforceApplication:
     """
     Base Jetforce application class with primitive URL routing.
 
-    This is a base class for writing jetforce server applications. It doesn't
+    This is a base class for writing jetforce server applications. It doesn't do
     anything on its own, but it does provide a convenient interface to define
     custom server endpoints using route decorators. If you want to utilize
     jetforce as a library and write your own server in python, this is the class
@@ -145,7 +144,7 @@ class JetforceApplication:
         try:
             request = Request(environ)
         except Exception:
-            send_status(Status.BAD_REQUEST, "Unrecognized URL format")
+            send_status(Status.BAD_REQUEST, "Invalid URL")
             return
 
         for route_pattern, callback in self.routes[::-1]:
@@ -197,13 +196,3 @@ class JetforceApplication:
         Set the error response based on the URL type.
         """
         return Response(Status.PERMANENT_FAILURE, "Not Found")
-
-    @classmethod
-    def add_arguments(cls, parser: argparse.ArgumentParser) -> None:
-        """
-        Add any application-specific arguments to the GeminiServer parser.
-
-        The destination variables for these arguments should match the method
-        signature for this class's __init__ method.
-        """
-        return
