@@ -7,6 +7,8 @@ import urllib.parse
 
 from twisted.internet.address import IPv4Address, IPv6Address
 from twisted.internet.defer import Deferred, ensureDeferred
+from twisted.internet.error import ConnectionClosed
+from twisted.internet.protocol import connectionDone
 from twisted.internet.task import deferLater
 from twisted.protocols.basic import LineOnlyReceiver
 
@@ -50,6 +52,7 @@ class GeminiProtocol(LineOnlyReceiver):
     def __init__(self, server: GeminiServer, app: JetforceApplication):
         self.server = server
         self.app = app
+        self._currently_deferred: typing.Optional[Deferred] = None
 
     def connectionMade(self):
         """
@@ -60,6 +63,14 @@ class GeminiProtocol(LineOnlyReceiver):
         self.response_buffer = ""
         self.client_addr = self.transport.getPeer()
 
+    def connectionLost(self, reason=connectionDone):
+        """
+        This is invoked by twisted after the connection has been closed.
+        """
+        if self._currently_deferred:
+            self._currently_deferred.errback(reason)
+            self._currently_deferred = None
+
     def lineReceived(self, line):
         """
         This method is invoked by LineOnlyReceiver for every incoming line.
@@ -67,6 +78,39 @@ class GeminiProtocol(LineOnlyReceiver):
         self.request = line
         return ensureDeferred(self._handle_request_noblock())
 
+    def lineLengthExceeded(self, line):
+        """
+        Called when the maximum line length has been reached.
+        """
+        return self.finish_connection()
+
+    def finish_connection(self):
+        """
+        Send the TLS "close_notify" alert and then immediately close the TCP
+        connection without waiting for the client to respond with it's own
+        "close_notify" alert.
+
+        > It is acceptable for an application to only send its shutdown alert
+        > and then close the underlying connection without waiting for the
+        > peer's response. This way resources can be saved, as the process can
+        > already terminate or serve another connection. This should only be
+        > done when it is known that the other side will not send more data,
+        > otherwise there is a risk of a truncation attack.
+
+        References:
+            https://github.com/michael-lazar/jetforce/issues/32
+            https://www.openssl.org/docs/man1.1.1/man3/SSL_shutdown.html
+        """
+        # Send the TLS close_notify alert and flush the write buffer. If the
+        # client has already closed their end of the stream, this will also
+        # close the underlying TCP connection.
+        self.transport.loseConnection()
+
+        # Ensure that the underlying connection will always be closed. There is
+        # no harm in calling this method twice if it was already invoked as
+        # part of the above TLS shutdown.
+        self.transport.transport.loseConnection()
+
     async def _handle_request_noblock(self):
         """
         Handle the gemini request and write the raw response to the socket.
@@ -95,34 +139,43 @@ class GeminiProtocol(LineOnlyReceiver):
             self.server.log_message(traceback.format_exc())
             self.write_status(Status.BAD_REQUEST, "Malformed request")
             self.flush_status()
-            self.transport.loseConnection()
+            self.finish_connection()
             raise
 
         try:
             environ = self.build_environ()
             response_generator = self.app(environ, self.write_status)
             if isinstance(response_generator, Deferred):
-                response_generator = await response_generator
+                response_generator = await self.track_deferred(response_generator)
             else:
                 # Yield control of the event loop
                 await deferLater(self.server.reactor, 0)
 
             for data in response_generator:
                 if isinstance(data, Deferred):
-                    data = await data
+                    data = await self.track_deferred(data)
                     self.write_body(data)
                 else:
                     self.write_body(data)
                     # Yield control of the event loop
                     await deferLater(self.server.reactor, 0)
 
+        except ConnectionClosed:
+            pass
         except Exception:
             self.server.log_message(traceback.format_exc())
             self.write_status(Status.CGI_ERROR, "An unexpected error occurred")
         finally:
             self.flush_status()
             self.log_request()
-            self.transport.loseConnection()
+            self.finish_connection()
+
+    async def track_deferred(self, deferred: Deferred):
+        self._currently_deferred = deferred
+        try:
+            return await deferred
+        finally:
+            self._currently_deferred = None
 
     def build_environ(self) -> typing.Dict[str, typing.Any]:
         """
